"""
SpkFunctor
负责对音频片段进行SPK说话人识别处理, 以SPK_Result进行callback
"""

from src.functor.base import BaseFunctor
from src.models import AudioBinary_Config, VAD_Functor_result, SpeakerCreate
from typing import Callable, List, Dict
from queue import Queue, Empty
import json
import torch
import threading
import numpy

# 日志
from src.utils.logger import get_module_logger

logger = get_module_logger(__name__)
from modelscope.pipelines import pipeline
# sv_pipeline = pipeline(
#     task='speaker-verification',
#     model='iic/speech_campplus_sv_zh-cn_16k-common',
#     model_revision='v1.0.0'
# )
# speaker1_a_wav = 'https://modelscope.cn/api/v1/models/damo/speech_campplus_sv_zh-cn_16k-common/repo?Revision=master&FilePath=examples/speaker1_a_cn_16k.wav'
# speaker1_b_wav = 'https://modelscope.cn/api/v1/models/damo/speech_campplus_sv_zh-cn_16k-common/repo?Revision=master&FilePath=examples/speaker1_b_cn_16k.wav'
# speaker2_a_wav = 'https://modelscope.cn/api/v1/models/damo/speech_campplus_sv_zh-cn_16k-common/repo?Revision=master&FilePath=examples/speaker2_a_cn_16k.wav'
# # 相同说话人语音
# result = sv_pipeline([speaker1_a_wav, speaker1_b_wav])
# print(result)
# # 不同说话人语音
# result = sv_pipeline([speaker1_a_wav, speaker2_a_wav])
# print(result)
# # 可以自定义得分阈值来进行识别，阈值越高，判定为同一人的条件越严格
# result = sv_pipeline([speaker1_a_wav, speaker1_a_wav], thr=0.6)
# print(result)
# # 可以传入output_emb参数，输出结果中就会包含提取到的说话人embedding
# result = sv_pipeline([speaker1_a_wav, speaker2_a_wav], output_emb=True)
# print(result['embs'], result['outputs'])
# # 可以传入save_dir参数，提取到的说话人embedding会存储在save_dir目录中
# result = sv_pipeline([speaker1_a_wav, speaker2_a_wav], save_dir='savePath/')

class SPKFunctor(BaseFunctor):
    """
    SPKFunctor
    负责对音频片段进行SPK说话人识别处理, 以SPK_Result进行callback
    需要配置好 _model, _callback, _input_queue, _audio_config
    否则无法run()启动线程

    运行中, 使用reset_cache()重置缓存, 准备下次任务

    使用stop()停止线程, 但需要等待input_queue为空
    """
    class speaker_verify:
        """
        说话人验证
        """
        def __init__(self, *args, **kwargs):
            self._spk_data: List[SpeakerCreate] = []

        def add_speaker(self, speaker: SpeakerCreate) -> None:
            self._spk_data.append(speaker)

        def verify(self, emb: numpy.ndarray) -> Dict:
            # 将输入的numpy embedding转换为tensor
            input_emb_tensor = torch.from_numpy(emb).unsqueeze(0)
            
            results = []
            for speaker in self._spk_data:
                if not speaker.speaker_embs:
                    continue
                # 将列表中存储的embedding转换为tensor
                speaker_emb_tensor = torch.tensor(speaker.speaker_embs).unsqueeze(0)
                
                # 计算余弦相似度
                score = torch.nn.functional.cosine_similarity(input_emb_tensor, speaker_emb_tensor).item()

                result = {
                    "score": score,
                    "speaker_id": speaker.speaker_id,
                    "speaker_name": speaker.speaker_name,
                    "speaker_description": speaker.speaker_description
                }
                results.append(result)

            if not results:
                return {
                    "speaker_id": "unknown",
                    "speaker_name": "Unknown",
                    "speaker_description": "No registered speakers to verify against.",
                    "score": 0.0,
                    "results": []
                }

            results.sort(key=lambda x: x['score'], reverse=True)
            best_match = results[0]
            return {
                "speaker_id": best_match['speaker_id'],
                "speaker_name": best_match['speaker_name'],
                "speaker_description": best_match['speaker_description'],
                "score": best_match['score'],
                "results": results
            }
            
    def __init__(self) -> None:
        super().__init__()
        # 资源与配置
        self._spk_verify = self.speaker_verify()
        self._sv_pipeline = pipeline(
            task='speaker-verification',
            model='iic/speech_campplus_sv_zh-cn_16k-common',
            model_revision='v1.0.0'
        )
        self._model: dict = {}  # 模型
        self._callback: List[Callable] = []  # 回调函数
        self._input_queue: Queue = None  # 输入队列
        self._audio_config: AudioBinary_Config = None  # 音频配置

    def load_spk_data_local(
        self,
        spk_data_path: str = 'data/speakers.json',
        ) -> None:
        """
        加载本地说话人数据
        """
        with open(spk_data_path, 'r') as f:
            spk_data = json.load(f)
            for spk in spk_data:
                self._spk_verify.add_speaker(SpeakerCreate(**spk))
        

    def reset_cache(self) -> None:
        """
        重置缓存, 用于任务完成后清理缓存数据, 准备下次任务
        """
        pass

    def set_input_queue(self, queue: Queue) -> None:
        """
        设置监听的输入消息队列
        """
        self._input_queue = queue

    def set_model(self, model: dict) -> None:
        """
        设置推理模型
        """
        self._model = model

    def set_audio_config(self, audio_config: AudioBinary_Config) -> None:
        """
        设置音频配置
        """
        self._audio_config = audio_config
        logger.debug("SpkFunctor设置音频配置: %s", self._audio_config)


    def add_callback(self, callback: Callable) -> None:
        """
        向自身的_callback: List[Callable]回调函数列表中添加回调函数
        """
        if not isinstance(self._callback, list):
            self._callback = []
        self._callback.append(callback)

    def _do_callback(self, result: List[str]) -> None:
        """
        回调函数
        """
        for callback in self._callback:
            callback(result)

    def _process(self, data: VAD_Functor_result) -> None:
        """
        处理数据
        """
        binary_data = data.audiobinary_data.binary_data
        # result = self._model["spk"].generate(
        #     input=binary_data,
        #     chunk_size=self._audio_config.chunk_size,
        # )
        sv_result = self._sv_pipeline([binary_data], output_emb=True)
        embs = sv_result['embs'][0]

        result = self._spk_verify.verify(embs)
        
        self._do_callback(result)

    def _run(self) -> None:
        """
        线程运行逻辑
        """
        with self._status_lock:
            self._is_running = True
            self._stop_event = False
        # 运行逻辑
        while self._is_running:
            try:
                data = self._input_queue.get(True, timeout=1)
                if data is None:
                    break
                logger.debug("[SPKFunctor]获取到的数据length: %s", len(data))
                self._process(data)
                self._input_queue.task_done()
            # 当队列为空时, 间隔1s检测是否进入停止事件。
            except Empty:
                if self._stop_event:
                    break
                continue
            # 其他异常
            except Exception as e:
                logger.error("SpkFunctor运行时发生错误: %s", e)
                raise e

    def run(self) -> threading.Thread:
        """
        启动线程
        Returns:
            threading.Thread: 返回已运行线程实例
        """
        self._pre_check()
        self._thread = threading.Thread(target=self._run, daemon=True)
        self._thread.start()
        return self._thread

    def _pre_check(self) -> bool:
        """
        预检查
        """
        if self._model is None:
            raise ValueError("模型未设置")
        if self._input_queue is None:
            raise ValueError("输入队列未设置")
        if self._callback is None:
            raise ValueError("回调函数未设置")
        return True

    def stop(self) -> bool:
        """
        停止线程
        """
        with self._status_lock:
            self._stop_event = True
        self._thread.join()
        with self._status_lock:
            self._is_running = False
        return not self._thread.is_alive()
