"""
src/schemas/speaker.py
------------------------
此模块定义与说话人（speakers）表对应的 Pydantic 模型，用于 API 数据验证和序列化。

模型说明：
- SpeakerBase：定义说话人的基础字段，包括姓名与描述。
- SpeakerCreate：用于创建说话人时的数据验证，直接继承 SpeakerBase。
- SpeakerUpdate：用于更新说话人信息时，所有字段均为可选。
- SpeakerResponse：返回给客户端时使用，包含数据库生成的字段（如 speaker_id、created_at 等）。
"""

from datetime import datetime
from typing import Optional, List
from uuid import UUID
from pydantic import BaseModel, Field
from .base import BaseSchema

# 基础模型，定义说话人的核心属性
class SpeakerBase(BaseSchema):
    speaker_id: UUID = Field(
        ..., 
        description="说话人唯一标识符"
    )
    speaker_name: str = Field(
        ..., 
        max_length=255, 
        description="说话人姓名，必填，最多255字符"
    )
    speaker_description: Optional[str] = Field(
        None, 
        description="说话人描述信息，可选"
    )
    speaker_embs: Optional[List[float]] = Field(
        None, 
        description="说话人embedding，可选"
    )

# 用于创建说话人时的数据模型，直接继承 SpeakerBase
class SpeakerCreate(SpeakerBase):
    pass

# 用于更新说话人时的数据模型，所有字段都是可选的
class SpeakerUpdate(BaseModel):
    speaker_name: Optional[str] = Field(
        None, 
        max_length=255, 
        description="说话人姓名"
    )
    speaker_description: Optional[str] = Field(
        None, 
        description="说话人描述信息"
    )

# 用于 API 返回的说话人响应模型，扩展基础模型以包含数据库生成的字段
class SpeakerResponse(SpeakerBase):
    speaker_id: UUID = Field(
        ..., 
        description="说话人的唯一标识符（UUID）"
    )
    created_at: datetime = Field(
        ..., 
        description="记录创建时间"
    )
    updated_at: datetime = Field(
        ..., 
        description="最近更新时间"
    )